/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_APPEARANCE_H
#define X3DTK_GL_APPEARANCE_H

#include "GL_X3DAppearanceNode.h"

namespace X3DTK {
namespace GL {

class X3DMaterialNode;
class X3DTextureNode;
class X3DTextureTransformNode;
class LineProperties;
class FillProperties;

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::Appearance.
 *
 *  \ingroup GLSceneGraph
 */
 
class Appearance : public X3DAppearanceNode
{
public:
  /// Constructor.
  Appearance();
  /// Destructor.
  virtual ~Appearance();
  
  /// Gets the Material attribute.
  inline X3DMaterialNode *getMaterial() {return _material;};
  /// Gets the Texture attribute.
  inline X3DTextureNode *getTexture() {return _texture;};
  /// Gets the TextureTransform attribute.
  inline X3DTextureTransformNode *getTextureTransform() {return _textureTransform;};
  /// Gets the LineProperties attribute.
  inline LineProperties *getLineProperties() {return _lineProperties;};
  /// Gets the FillProperties attribute.
  inline FillProperties *getFillProperties() {return _fillProperties;};

private:  
  /// Material node.
  X3DMaterialNode *_material;
  /// Texture node.
  X3DTextureNode *_texture;
  /// Texture transform node.
  X3DTextureTransformNode *_textureTransform;
  /// Line properties node.
  LineProperties *_lineProperties;
  /// FIll properties node.
  FillProperties *_fillProperties;
};

}
}

#endif
