/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_IMAGETEXTURE_H
#define X3DTK_GL_IMAGETEXTURE_H

#include "GL_X3DTexture2DNode.h"
#include <map>

namespace X3DTK {
namespace GL {

struct TexInfo
{
  GLuint texName;
  float textureTransform[16];
  unsigned short refCount;
};

class ImageTexture;

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::ImageTexture.
 *
 *  \ingroup GLSceneGraph
 */
 class ImageTexture : public X3DTexture2DNode
{
public:
  /// Constructor.
  ImageTexture();
  /// Constructor.
  virtual ~ImageTexture();
  
  /// Gets the texName.
  inline const GLuint &getTexName() const {return _texName;};
  /// Gets the ratioS.
  const float (&getTextureTransform() const)[16] {return _textureTransform;};

  /// Updates the attributes of the node.
  virtual void update();

private:  
  /// Texture name attribute.
  GLuint _texName;
  static std::map<SFString, TexInfo> _texNameMap;
  /// Texture transform attribute.
  float _textureTransform[16];
};

}
}

#endif
