/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_GL_INDEXEDLINESET_H
#define X3DTK_GL_INDEXEDLINESET_H

#include "GL_X3DGeometryNode.h"

#include <vector>

namespace X3DTK {
namespace GL {

/*! \brief Class declared in <b><X3DTK/GL/scenegraph.h></b> providing the GL implementation
 *  of X3D::IndexedLineSet.
 *
 *  \ingroup GLSceneGraph
 */
 
class IndexedLineSet : public X3DGeometryNode
{
public:
  /// Constructor.
  IndexedLineSet();
  /// Destructor.
  virtual ~IndexedLineSet();

  /// Sets the line width attribute.
  void setLineWidth(const SFFloat &lineWidth);
  /// Sets the verticesDuplicated attribute.
  void setVerticesDuplicated(const SFBool &verticesDuplicated);
  /// Sets the color attribute.
  void setColor(const SFBool &color);

  /// Sets the C4UB_V3F vertex array.
  void setC4UB_V3F_vertexArray(const std::vector<C4UB_V3F> &C4UB_V3F_vertexArray);   
  /// Sets the C4UB_V3F vertex array.
  void setV3F_vertexArray(const std::vector<SFVec3f> &V3F_vertexArray);
  /// Sets the index array.
  void setIndexArrayArray(const std::vector<MFInt32> &indexArrayArray);
  
  /// Gets the Line width attribute.
  inline const SFFloat &getLineWidth() const {return _lineWidth;};
  /// Gets the color per vertex attribute.
  inline const SFBool &getVerticesDuplicated() const {return _verticesDuplicated;};
  /// Gets the color attribute.
  inline const SFBool &getColor() const {return _color;};

  /// Gets the C4UB_V3F vertex array.
  inline std::vector<C4UB_V3F> &C4UB_V3F_vertexArray() {return _C4UB_V3F_vertexArray;};   
  /// Gets the C4UB_V3F vertex array.
  inline std::vector<SFVec3f> &V3F_vertexArray() {return _V3F_vertexArray;};
  /// Gets the index array.  
  inline std::vector<MFInt32> &indexArrayArray() {return _indexArrayArray;};

  /// Updates the attributes of the node.
  virtual void update();
  /// Renders the node in an openGL context.
  virtual void draw() const;  

private:  
  /// Line width attribute.
  SFFloat _lineWidth;
  /// Color per vertex attribute.
  SFBool _verticesDuplicated;
  /// Color attribute.
  SFBool _color;
  /// Array of vertex arrays.
  std::vector<C4UB_V3F> _C4UB_V3F_vertexArray;
  /// Array of vertex arrays.
  std::vector<SFVec3f> _V3F_vertexArray;
  /// Array of Line index.
  std::vector<MFInt32> _indexArrayArray;
};

}
}

#endif
