/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_MESHSIMPLIFIERSTATEVARIABLES_H
#define X3DTK_MESH_MESHSIMPLIFIERSTATEVARIABLES_H

#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"
#include "MESH_Vertex.h"

#include <list>
#include <vector>
#include <map>

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/simplemeshbuilder.h></b> providing the StateVariables
 *  of the MESH::TemplateMeshSimplifier processor.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateMeshSimplifierStateVariables : public StateVariables
{
public:
  /// Constructor.
  TemplateMeshSimplifierStateVariables();

  /// Initializes the traversal.
  void init();
  /// Finishes the traversal.
  void finish();
   
  /// Adds the Coordinate.
  void addNode(SFNode N);
  /// Returns true if Coordinate has already been visited.
  bool getNode(SFNode N) const;
  
  /// Returns the built Mesh.
  inline TemplateMesh<MData, VData, EData, FData, RW> *getMesh() const {return _mesh;};
  
  /// Adds the couple vertex.
  void addCoupleVertex(SFTemplateVertex<MData, VData, EData, FData, RW> *A, SFTemplateVertex<MData, VData, EData, FData, RW> *B);
  /// Returns the index of the vertex.
  SFTemplateVertex<MData, VData, EData, FData, RW> *getNode(SFTemplateVertex<MData, VData, EData, FData, RW> *V) const;
  
private:  
  std::list<SFNode> _nodeList;
  TemplateMesh<MData, VData, EData, FData, RW> *_mesh;
  std::map<SFTemplateVertex<MData, VData, EData, FData, RW> *, SFTemplateVertex<MData, VData, EData, FData, RW> *> _vertexMap;
};

}
}

#include "MESH_MeshSimplifierStateVariables.inl"

#endif
