/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_NORMALSDRAWER_H
#define X3DTK_MESH_NORMALSDRAWER_H

#include "MESH_NormalsDrawerStateVariables.h"
#include "DFSGraphTraversal.h"
#include "MESH_NormalsDrawerCoreVisitor.h"
#include "X3DOnePassProcessor.h"
#include "MESH_SceneGraphTypes.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/normalsdrawer.h></b> providing a processor that 
 *  draws the normals of a MESH scene graph.
 *
 *  The amplitude of the normals can be modulated.
 *
 *  \ingroup MESH
 */
 
template<class MData, class VData, class EData, class FData, bool RW>
class TemplateNormalsDrawer : public X3DOnePassProcessor
{
public:
  /// Constructor.
  TemplateNormalsDrawer();
  /// Destructor.
  virtual ~TemplateNormalsDrawer();
  
  /// Draws the normals.
  void draw(X3DNode *N, float amplitude = 1.0f);
};

/// Default TemplateNormalsDrawer.
typedef TemplateNormalsDrawer<MeshData, VertexData, EdgeData, FaceData, true> NormalsDrawer;

}
}

#include "MESH_NormalsDrawer.inl"

#endif

