/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_MESH_PRIMITIVEMESHBUILDER_H
#define X3DTK_MESH_PRIMITIVEMESHBUILDER_H

#include "X3DOnePassProcessor.h"
#include "MESH_SceneGraphTypes.h"
#include "MESH_Mesh.h"
#include "MESH_Scene.h"
#include "MESH_MeshData.h"

namespace X3DTK {
namespace MESH {

/*! \brief Class declared in <b><X3DTK/MESH/PrimitiveMeshBuilder.h></b> providing a processor that 
 *  builds the meshes from the geometric primitives (Cone, Cylinder, Box and Sphere) of the MESH scene graph.
 *
 *  The discretization can be chosen. The step can be fixed or function of the bounding box.
 *
 *  \ingroup MESH
 */

class PrimitiveMeshBuilder : public X3DOnePassProcessor
{
public:
  /// Constructor.
  PrimitiveMeshBuilder();
  /// Destructor.
  virtual ~PrimitiveMeshBuilder();
  
  /// Builds the meshes from the geometric primitives of the scene.
  void build(Scene *S, float step = 0.0f, bool boundingBox = false);
  /// Builds the meshes from the geometric primitives of the sub-graph.
  void build(X3DNode *N, float step = 0.0f, bool boundingBox = false);
};

}
}

#endif
