/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef SFSCENEGRAPH_H
#define SFSCENEGRAPH_H

#include "X3DTypes.h"

namespace X3DTK {

/*! \brief Class declared in <b><X3DTK/kernel.h></b> storing the informations relative 
 *  to the scene graphs of the inheritance tree. 
 *
 *  It is based upon one singleton per tag. It means that it 
 *  exists only one instance of SFSceneGraph of name "GL". 
 *
 *  \ingroup abstractNode
 */

class SFSceneGraph
{
  friend class SFComponent;
  
public:
  /// Gets the SFSceneGraph by its name.
  static SFSceneGraph *getSceneGraph(const SFString &name);
  
  /// Returns the name of the scene graph.
  inline const SFString &getName() const {return _name;};
  /// Gets the list of the SFType.
  inline const MFComponent &getComponentList() const {return _componentList;};

private:  
  SFSceneGraph(const SFString &name);
  ~SFSceneGraph();
  
  /// Adds one SFType.
  void addComponent(SFComponent *component);
  /// Adds one reference.
  void addOneReference();
  /// Removes one reference of the SFSceneGraph.
  static void removeOneReference(SFSceneGraph *sceneGraph);
  
  SFString _name;    
  int _refCount;
  MFComponent _componentList;
  
  static MFSceneGraph _sceneGraphList;
};

}

#endif
