/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_SINGLETON_H
#define X3DTK_SINGLETON_H

namespace X3DTK {

/*! \brief Template class declared in <b><X3DTK/kernel.h></b> defining a Singleton
 *  manager. 
 *  
 *  It simply gets the instance and remove the instance. A template singleton manager
 *  avoids the user to implement the pattern. In the library, it is commonly used
 *  with X3DTK::X3DProcessor and X3DTK::StateVariables.
 *
 * \ingroup processor
 */

template<class T>
class Singleton
{
public:
  /// Gets the instance. If no instance is created, then a new one is created.
  static T *getInstance();
  /// Removes the instance.
  static void removeInstance();

private:
  static T *_instance;  
};

}

#include "Singleton.inl"

#endif
