/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_BBOXUPDATER_H
#define X3DTK_X3D_BBOXUPDATER_H

#include "X3D_BBoxUpdaterStateVariables.h"
#include "X3DOnePassProcessor.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/bboxupdater.h></b> providing a processor
 *  that computes the bounding boxes of the X3D scene graph.
 *
 *  The bounding boxes are mainly stored in the X3DTK::X3D::Shape, X3DTK::X3D::Group, 
 *  X3DTK::X3D::Transform, X3DTK::X3D::Scene which inherit X3DTK::X3D::X3DBoundedObject.
 *  To get the bounding box attribute, you can write:
 *  
 *  \code
 *  X3DTK::X3D::Shape *S;
 *  // Getting the center of the bounding box.
 *  const SFVec3f &center = S->getCenter();
 *  // Getting the diagonal vector of the bounding box.
 *  const SFVec3f &size = S->getSize();
 *  \endcode
 *
 *  \ingroup X3D
 */

class BBoxUpdater : public X3DOnePassProcessor
{
public:
  /// Constructor.
  BBoxUpdater();
  /// Destructor.
  virtual ~BBoxUpdater();
  
  /// Updates the Bounding Boxes of the scene.
  virtual void update(SFNode N, bool staticProcessing = false);
};

}
}

#endif
