/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_COORDMERGER_H
#define X3DTK_X3D_COORDMERGER_H

#include "X3D_SceneGraphTypes.h"
#include "X3D_CoordMergerStateVariables.h"
#include "X3DOnePassProcessor.h"

namespace X3DTK {
namespace X3D {

class Scene;

/*! \brief Class declared in <b><X3DTK/X3D/coordmerger.h></b> providing a processor
 *  that merges identical values of the Coordinate nodes . 
 *
 *  The processor edits the X3DTK::X3D::Coordinate nodes by merging identical values
 *  and updating the coordIndex attribute of the X3DTK::X3D::IndexedFaceSet parents.
 *
 *  \ingroup X3D
 */

class CoordMerger : public X3DOnePassProcessor
{
public:
  /// Constructor.
  CoordMerger();
  /// Destructor.
  virtual ~CoordMerger();

  /// Merges identical values.
  virtual void merge(Scene *S);
  /// Merges identical values.
  virtual void merge(X3DNode *N);
};

}
}

#endif
