/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_FILLPROPERTIES_H
#define X3DTK_X3D_FILLPROPERTIES_H

#include "X3DTypes.h"
#include "X3D_X3DAppearanceChildNode.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b>, part of the <a href="../../doc/X3DSpecs/Part01/components/shape.html">shape 
 *  component</a> defining a FillProperties
 *  (<a href="../../doc/X3DSpecs/Part01/components/shape.html#FillProperties">
 *  documentation</a>).
 *
 *  \ingroup X3DSceneGraph
 */

class FillProperties : public X3DAppearanceChildNode
{
public:
  /// Constructor.
  FillProperties();
  /// Destructor.
  virtual ~FillProperties();
  
  /// Sets the FillStyle attribute.
  void setFillStyle(const SFString &fillStyle);
  /// Sets the HatchStyle attribute.
  void setHatchStyle(const SFInt32 &hatchStyle);
  /// Sets the HatchColor attribute.
  void setHatchColor(const SFColor &hatchColor);
  
  /// Sets the FillStyle attribute.
  inline const SFString &getFillStyle() const {return _fillStyle;};
  /// Sets the HatchStyle attribute.
  inline const SFInt32 &getHatchStyle() const {return _hatchStyle;};
  /// Sets the HatchColor attribute.
  inline const SFColor &getHatchColor() const {return _hatchColor;};

private:
  /// Fill style.
  SFString _fillStyle;
  /// Hatch style.
  SFInt32 _hatchStyle;
  /// Hatch color.
  SFColor _hatchColor;
};

}
}

#endif
