/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3D_SCENESAVER_H
#define X3DTK_X3D_SCENESAVER_H

#include "X3D_SceneSaverStateVariables.h"
#include "X3DMultiPassProcessor.h"

namespace X3DTK {
namespace X3D {

class Scene;

/*! \brief Class declared in <b><X3DTK/X3D/sceneaver.h></b> providing a processor
 *  that saves the content of an X3D scene graph into a file.
 *
 *  The processor uses the X3DTK::X3D::X3DNode::write method to write the content of
 *  the nodes. If the scene graph contains shared nodes which have no name (DEF attribute), 
 *  then an arbitratry one is assigned.
 * 
 *  \ingroup X3D
 */
 
class SceneSaver : public X3DMultiPassProcessor
{
public:
  /// Constructor.
  SceneSaver();
  /// Destructor.
  virtual ~SceneSaver();
  
  /// Writes the content of S in file.
  virtual void saveAs(Scene *S, const SFString &fileName);
  /// Writes the content of S in file.
  virtual void save(Scene *S);
};

}
}

#endif
