/// This file is part of the X3DToolKit library
/// Copyright (C) 2002-2004  Yannick Le Goc (legoc@imag.fr)

/// http://artis.imag.fr/Members/Yannick.Legoc/X3D/

/// This library is free software; you can redistribute it and/or
/// modify it under the terms of the GNU Lesser General Public
/// License as published by the Free Software Foundation; either
/// version 2.1 of the License, or (at your option) any later version.

/// This library is distributed in the hope that it will be useful,
/// but WITHOUT ANY WARRANTY; without even the implied warranty of
/// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
/// Lesser General Public License for more details.

/// You should have received a copy of the GNU Lesser General Public
/// License along with this library; if not, write to the Free Software
/// Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
#ifndef X3DTK_X3DCOMPONENTCREATOR_H
#define X3DTK_X3DCOMPONENTCREATOR_H

#include "X3DComponent.h"
#include "Recorder.h"

namespace X3DTK {
namespace X3D {

/*! \brief Class declared in <b><X3DTK/X3D/scenegraph.h></b> defining a component creator
 *  used by the SAX2 loader to find the good creation function.
 *  
 *  \ingroup X3DLoader
 */

class X3DComponentCreator : public X3DComponent
{
public:
  /// Constructor.
  X3DComponentCreator();
  /// Destructor.
  virtual ~X3DComponentCreator() = 0;
  
  /// Gets the creation of name.
  CreationFunction *getCreationFunctionOf(const SFString &name) const;
  /// Returns the list of functions
  inline CreationDict getCreationDict() const {return _creationDict;};
  
  /// Returns TRUE if the component contains the type name.
  bool contains(const SFString &name) const;
  
protected:
  /// Defines a node of type T.
  template<class T>
  void defineNode();
  
  /// Defines a node of type T.
  void define(const std::pair<StringType, CreationFunction *> &entry);

private:
  CreationDict _creationDict;
};

}
}

#include "X3D_X3DComponentCreator.inl"

#endif
